@@ -20,19 +20,14 @@
 #endif
 
 #define MYSQL_SERVER 1
-#include <my_global.h>
+#include "heapdef.h"
 #include "sql_priv.h"
 #include "sql_plugin.h"
 #include "ha_heap.h"
-#include "heapdef.h"
 #include "sql_base.h"                    // enum_tdc_remove_table_type
 
-static handler *heap_create_handler(handlerton *hton,
-                                    TABLE_SHARE *table, 
-                                    MEM_ROOT *mem_root);
-static int
-heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
-                            HP_CREATE_INFO *hp_create_info);
+static handler *heap_create_handler(handlerton *, TABLE_SHARE *, MEM_ROOT *);
+static int heap_prepare_hp_create_info(TABLE *, bool, HP_CREATE_INFO *);
 
 
 int heap_panic(handlerton *hton, ha_panic_function flag)
@@ -251,7 +246,7 @@ int ha_heap::write_row(uchar * buf)
   return res;
 }
 
-int ha_heap::update_row(const uchar * old_data, uchar * new_data)
+int ha_heap::update_row(const uchar * old_data, const uchar * new_data)
 {
   int res;
   res= heap_update(file,old_data,new_data);
@@ -369,7 +364,7 @@ int ha_heap::info(uint flag)
   HEAPINFO hp_info;
 
   if (!table)
-    return 1;
+    return 0;
 
   (void) heap_info(file,&hp_info,flag);
 
@@ -431,6 +426,10 @@ int ha_heap::reset_auto_increment(ulonglong value)
 
 int ha_heap::external_lock(THD *thd, int lock_type)
 {
+#ifndef DBUG_OFF
+  if (lock_type == F_UNLCK && file->s->changed && heap_check_heap(file, 0))
+    return HA_ERR_CRASHED;
+#endif
   return 0;					// No external locking
 }
 
@@ -600,16 +599,15 @@ ha_rows ha_heap::records_in_range(uint inx, key_range *min_key,
 }
 
 
-static int
-heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
-                            HP_CREATE_INFO *hp_create_info)
+static int heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
+                                       HP_CREATE_INFO *hp_create_info)
 {
-  uint key, parts, mem_per_row= 0, keys= table_arg->s->keys;
+  TABLE_SHARE *share= table_arg->s;
+  uint key, parts, mem_per_row= 0, keys= share->keys;
   uint auto_key= 0, auto_key_type= 0;
   ha_rows max_rows;
   HP_KEYDEF *keydef;
   HA_KEYSEG *seg;
-  TABLE_SHARE *share= table_arg->s;
   bool found_real_auto_increment= 0;
 
   bzero(hp_create_info, sizeof(*hp_create_info));
@@ -617,11 +615,11 @@ heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
   for (key= parts= 0; key < keys; key++)
     parts+= table_arg->key_info[key].user_defined_key_parts;
 
-  if (!(keydef= (HP_KEYDEF*) my_malloc(keys * sizeof(HP_KEYDEF) +
-				       parts * sizeof(HA_KEYSEG),
-				       MYF(MY_WME | MY_THREAD_SPECIFIC))))
+  if (!my_multi_malloc(MYF(MY_WME | MY_THREAD_SPECIFIC),
+                       &keydef, keys * sizeof(HP_KEYDEF),
+                       &seg, parts * sizeof(HA_KEYSEG),
+                       NULL))
     return my_errno;
-  seg= reinterpret_cast<HA_KEYSEG*>(keydef + keys);
   for (key= 0; key < keys; key++)
   {
     KEY *pos= table_arg->key_info+key;
@@ -781,6 +779,54 @@ bool ha_heap::check_if_incompatible_data(HA_CREATE_INFO *info,
   return COMPATIBLE_DATA_YES;
 }
 
+
+/**
+  Find record by unique index (used in temporary tables with the index)
+
+  @param record          (IN|OUT) the record to find
+  @param unique_idx      (IN) number of index (for this engine)
+
+  @note It is like hp_search but uses function for raw where hp_search
+        uses functions for index.
+
+  @retval  0 OK
+  @retval  1 Not found
+  @retval -1 Error
+*/
+
+int ha_heap::find_unique_row(uchar *record, uint unique_idx)
+{
+  DBUG_ENTER("ha_heap::find_unique_row");
+  HP_SHARE *share= file->s;
+  DBUG_ASSERT(inited==NONE);
+  HP_KEYDEF *keyinfo= share->keydef + unique_idx;
+  DBUG_ASSERT(keyinfo->algorithm == HA_KEY_ALG_HASH);
+  DBUG_ASSERT(keyinfo->flag & HA_NOSAME);
+  if (!share->records)
+    DBUG_RETURN(1); // not found
+  HASH_INFO *pos= hp_find_hash(&keyinfo->block,
+                               hp_mask(hp_rec_hashnr(keyinfo, record),
+                                       share->blength, share->records));
+  do
+  {
+    if (!hp_rec_key_cmp(keyinfo, pos->ptr_to_rec, record))
+    {
+      file->current_hash_ptr= pos;
+      file->current_ptr= pos->ptr_to_rec;
+      file->update = HA_STATE_AKTIV;
+      /*
+        We compare it only by record in the index, so better to read all
+        records.
+      */
+      memcpy(record, file->current_ptr, (size_t) share->reclength);
+
+      DBUG_RETURN(0); // found and position set
+    }
+  }
+  while ((pos= pos->next_key));
+  DBUG_RETURN(1); // not found
+}
+
 struct st_mysql_storage_engine heap_storage_engine=
 { MYSQL_HANDLERTON_INTERFACE_VERSION };
 
